%Estimating Nonseparable Models with Mismeasured Endogenous Variables
%  by Suyong Song, Susanne M. Schennach and Halbert White
% Quantitative Economics (forthcoming)
%
%This code generates Table 6: Impact of family income on children's math achievement
%This code estimates standard devidations by bootstrap


clear all;
close all;
clc;
   
tic


NUMERIC = xlsread('data_curr_math'); %upload the data

y = NUMERIC(:,2);
x = log(NUMERIC(:,33));
z = log(NUMERIC(:,34));
w = NUMERIC(:,14);

ave = (mean(x) + mean(z))/2;
sd = (std(x)^2 + std(z)^2)/2;

y = (y-mean(y))/std(y);
x = (x-ave)/sd;
z = (z-ave)/sd;
w = (w-mean(w))/std(w);


n=length(y); % number of observations
nb=1024;  %number of mashes for the numerical integral
wm=16;
 
%Bandwidths selections
h1=6.8;
h2=5.7;
h3=6.8;
h4=7;

mid=0;
wid=1.0;
minx=mid-wid;
maxx=mid+wid;
widx=maxx-minx;

[lz, dz]=set_mesh(minx,maxx,nb,wm);
xs=(lz:dz:lz+dz*(nb-1))';
ws=xs;

bb=500; % number of bootstrapping

    f_hat1 = zeros(n,1);
    f_hat3 = zeros(n,1);
    
    for ii =1:n
    
    thex=x(ii,1);
    thew=w(ii,1);
    [f_x, muhat1, betahat1]=nlme(y,x,z,w,h1,h2,nb,lz,dz,thew);
    f_hat1(ii,1)=Ey_x(muhat1,lz,dz,thex);
            
    kx = ftkernel2(x-thex,h3);
    kw = ftkernel2(w-thew,h4);        
    k2 = calc_diag(n,h3,h4,kx,kw);
    chi2 = [ones(n,1) (x-thex) (w-thew)];    
    muhat3=inv(chi2'*k2*chi2)*chi2'*k2*y;
    f_hat3(ii,1)=muhat3(1);
    
    end
    
    u1 = y - f_hat1;
    utilde1 = u1 - mean(u1);
    
    u3 = y - f_hat3;
    utilde3 = u3 - mean(u3);
    
thex_v=-2.8:0.2:1.6;  %given point of x
thew_v=1.2:0.2:1.2;  %given point of w

m1=length(thex_v);
m2=length(thew_v);
std_vec1 = zeros(m2,m1);
std_vec3 = zeros(m2,m1);

for p =1:m2
    
    theww=thew_v(p);

    for q=1:m1
        
        thexx=thex_v(q);
    
    curyh1=[];
    curyh3=[];
    boot=1;
    while boot <= bb
    
    ustar1=zeros(n,1);
    ustar3=zeros(n,1);
    
	for j=1:n

		idx=ceil((n)*rand(1,1));
		ustar1(j,1)=utilde1(idx,1);
        ustar3(j,1)=utilde3(idx,1);
	
    end
    
    ystar1 = f_hat1 + ustar1;
    ystar3 = f_hat3 + ustar3;

    [f_x, muhat1, betahat1]=nlme(ystar1,x,z,w,h1,h2,nb,lz,dz,theww);
    yh1=Ey_x(betahat1,lz,dz,thexx);
    curyh1=[curyh1; yh1];
    kkx = ftkernel2(x-thexx,h3);
    kkw = ftkernel2(w-theww,h4);
    kk2 = calc_diag(n,h3,h4,kkx,kkw);
    yh3=inv(chi2'*kk2*chi2)*chi2'*kk2*ystar3;
    curyh3=[curyh3; yh3(2)];    
    boot =boot+1;
        
    end
    
    sd1=std(curyh1);
    std_vec1(p,q)=sd1;
    
    sd3=std(curyh3);
    std_vec3(p,q)=sd3;
   
end

end

toc

