%Estimating Nonseparable Models with Mismeasured Endogenous Variables
%  by Suyong Song, Susanne M. Schennach and Halbert White
% Quantitative Economics (forthcoming)
%
%Table 3. Monte Carlo simulation results as a function of sample size


clear all;
close all;
clc;

tic;

g_vec1=[];
g_vec2=[];

cases=[0 0 0 0 1; 
       0 0 0 0 0; 
       0 0 1 0 0;
       0 0 1 0 1;
       0 1 0 0 0;
       0 1 0 0 1;
       0 1 1 0 0;
       0 1 1 0 1;
       1 0 0 0 0;
       1 0 0 0 1;
       1 0 1 0 0;
       1 0 1 0 1;
       1 1 0 0 0;
       1 1 0 0 1;
       1 1 1 0 0;
       1 1 1 0 1];  

c=1;

%Bandwidths selections
h1min=6.5;   
h1max=8; 
h2min=4;  
h2max=5.5; 
dh1=0.05;
dh2=0.05;
h1_vec=(h1min:dh1:h1max)';
h2_vec=(h2min:dh2:h2max)';
m1=length(h1_vec);
m2=length(h2_vec);

        
n=1000; % number of observations
nb=1024; %number of mashes for the numerical integral
wm=16;
nrep=500; % number of simulations

mid=0;
wid=1.0;
minx=mid-wid;
maxx=mid+wid;
widx=maxx-minx;

thex=0;   %a given point of x
thew=1.0;   %a given point of w

[lz, dz]=set_mesh(minx,maxx,nb,wm);
xs=(lz:dz:lz+dz*(nb-1))';
ws=xs;

ys2=true_g(ws,cases(c,5));

sdx=0.5;

b_temp1=zeros(m1,m2);
v_temp1=zeros(m1,m2);
m_temp1=zeros(m1,m2);

index=[];

cur=length(index)+1;
while length(index) <= nrep
   
    if cases(c,2)==0
        w=1.0*randn(n,1);
    else
        w=1.0*rndlaplace(n,1);
    end   
    
    if cases(c,1)==0
        x_e= 0.5*w + sqrt(sdx)*randn(n,1);  
    else
        x_e= 0.5*w + sqrt(sdx)*rndlaplace(n,1);
    end
    
    
y_e=true_g(x_e,cases(c,4)).*true_g(w,cases(c,5)); % standardized to have mean zero and s.d. one

if cases(c,3)==0
    x=x_e+sdx*randn(n,1);
    z=x_e+sdx*randn(n,1);
else
    x=x_e+sdx*rndlaplace(n,1);
    z=x_e+sdx*rndlaplace(n,1);
end

u=true_g(w,cases(c,5)) + 0.3*randn(n,1); 

y = true_g(x_e,cases(c,4)).*u;
E_y = mean(y);
SD_y = std(y);
y=(true_g(x_e,cases(c,4)).*u -E_y)./SD_y;

if cases(c,4)==0
    true_beta = ( 2/sqrt(pi)*exp(-thex^2).*Ey_x(ys2,lz,dz,thew))./SD_y;
else
    true_beta = ( (thex >= -1)*(thex <=1).*Ey_x(ys2,lz,dz,thew))./SD_y;
end


for i=1:m1
    h1=h1_vec(i);
    h3=h1-3;
    h5=h1-3.5;
    h7=h1;
    h9=h1;
    
    for ii=1:m2
        h2=h2_vec(ii);
        h4=h2-2.5;
        h6=h2-2.5;
        h8=h2;
        h10=h2;
    
    [f_x1, muhat1, betahat1]=nlme(y,x,z,w,h1,h2,nb,lz,dz,thew);
    yh1=Ey_x(betahat1,lz,dz,thex);
       
    
    if sum(isnan(yh1))> 0 | max(abs(yh1)') > 50 
  
    continue
    end

    b_temp1(i,ii)= b_temp1(i,ii)+yh1/nrep;
    v_temp1(i,ii)= v_temp1(i,ii)+yh1^2/nrep;
    m_temp1(i,ii)= m_temp1(i,ii)+(yh1 - true_beta)^2/nrep;
     
    end
end

index=[index;cur];

cur=cur+1;

end

bias_f1 = (b_temp1 - true_beta).^2;
var_f1 = v_temp1 - b_temp1.^2;
rmse_f1 = sqrt(bias_f1+var_f1);


toc
