%Estimating Nonseparable Models with Mismeasured Endogenous Variables
%  by Suyong Song, Susanne M. Schennach and Halbert White
% Quantitative Economics (forthcoming)
%
%This code generates Table 6: Impact of family income on children's math achievement
%This code estimates covariate-conditioned average marginal effects
%It also plots Figures 2-3


clear all;
close all;
clc;

NUMERIC = xlsread('data_curr_math'); %upload the data

y = NUMERIC(:,2);
x = log(NUMERIC(:,33));
z = log(NUMERIC(:,34));
w = NUMERIC(:,14);

ave = (mean(x) + mean(z))/2;
sd = (std(x)^2 + std(z)^2)/2;

y = (y-mean(y))/std(y);
x = (x-ave)/sd;
z = (z-ave)/sd;
w = (w-mean(w))/std(w);

thex_v=-2.8:0.2:1.6;  %given point of x
thew_v=-1.2:0.2:1.2;  %given point of w

m1=length(thex_v);
m2=length(thew_v);
beta_vec1 = zeros(m2,m1);
mu_vec1 = zeros(m2,m1);
beta_vec3 = zeros(m2,m1);
mu_vec3 = zeros(m2,m1);

for ii =1:m2
    
    thew=thew_v(ii);
    
    for iii=1:m1
        
        thex=thex_v(iii);

n=length(y); % number of observations
nb=1024; %number of mashes for the numerical integral
wm=16;

%Bandwidths selections
h1=6.8; 
h2=5.7;
h3=6.8;
h4=7;

mid=0;
wid=1.0;
minx=mid-wid;
maxx=mid+wid;
widx=maxx-minx;

[lz, dz]=set_mesh(minx,maxx,nb,wm);
xs=(lz:dz:lz+dz*(nb-1))';
ws=xs;

    [f_x, muhat, betahat]=nlme(y,x,z,w,h1,h2,nb,lz,dz,thew);
    yh1=Ey_x(betahat,lz,dz,thex);
    yh2=Ey_x(muhat,lz,dz,thex);
     
    kx =  ftkernel2(x-thex,h3);
    kw =  ftkernel2(w-thew,h4);
         
    k2 = calc_diag(n,h3,h4,kx,kw);
    chi2 = [ones(n,1) (x-thex) (w-thew)];    
    yh3=inv(chi2'*k2*chi2)*chi2'*k2*y;
    
beta_vec1(ii,iii)=yh1;
mu_vec1(ii,iii)=yh2;
beta_vec3(ii,iii) =yh3(2);
mu_vec3(ii,iii)=yh3(1);  

end

end


mesh(thex_v,thew_v,beta_vec1);

xlabel('Income'), ylabel('AFQT'), zlabel('Average Marginal Effects');
title('Figure 2a: Impact of Family Income on Math Scores');

figure(2);
mesh(thex_v,thew_v,mu_vec1);

xlabel('Income'), ylabel('AFQT'), zlabel('Average Response');
title('Figure 2b: Conditional Expectaion or Average Counterfactual Response');

figure(3);
mesh(thex_v,thew_v,beta_vec3);

xlabel('Income'), ylabel('AFQT'), zlabel('Average Marginal Effects');
title('Figure 3a: Impact of Family Income on Math Scores');

figure(4);
mesh(thex_v,thew_v,mu_vec3);

xlabel('Income'), ylabel('AFQT'), zlabel('Average Response');
title('Figure 3b: Conditional Expectaion or Average Counterfactual Response');

