%Estimating Nonseparable Models with Mismeasured Endogenous Variables
%  by Suyong Song, Susanne M. Schennach and Halbert White
% Quantitative Economics (forthcoming)
%
%For Table 2: Monte Carlo simulation results


clear all;
close all;
clc;

tic;

g_vec1=[];
g_vec2=[];

cases=[0 0 0 0 1; 
       0 0 0 0 0; 
       0 0 1 0 0;
       0 0 1 0 1;
       0 1 0 0 0;
       0 1 0 0 1;
       0 1 1 0 0;
       0 1 1 0 1;
       1 0 0 0 0;
       1 0 0 0 1;
       1 0 1 0 0;
       1 0 1 0 1;
       1 1 0 0 0;
       1 1 0 0 1;
       1 1 1 0 0;
       1 1 1 0 1];  

c=1;

%Bandwidths selections
h1min=9.5;   
h1max=11;   
h2min=4;  
h2max=5.5;  
dh1=0.05;
dh2=0.05;
h1_vec=(h1min:dh1:h1max)';
h2_vec=(h2min:dh2:h2max)';
m1=length(h1_vec);
m2=length(h2_vec);

        
n=1000; % number of observations
nb=1024; %number of mashes for the numerical integral
wm=16;
nrep=500; % number of simulations

mid=0;
wid=1.0;
minx=mid-wid;
maxx=mid+wid;
widx=maxx-minx;

thex=0;   %a given point of x
thew=1.0;   %a given point of w

[lz, dz]=set_mesh(minx,maxx,nb,wm);
xs=(lz:dz:lz+dz*(nb-1))';
ws=xs;

ys2=true_g(ws,cases(c,5));

sdx=0.5;

b_temp1=zeros(m1,m2);
b_temp2=zeros(m1,m2);
b_temp3=zeros(m1,m2);
b_temp4=zeros(m1,m2);
b_temp5=zeros(m1,m2);
v_temp1=zeros(m1,m2);
v_temp2=zeros(m1,m2);
v_temp3=zeros(m1,m2);
v_temp4=zeros(m1,m2);
v_temp5=zeros(m1,m2);
m_temp1=zeros(m1,m2);
m_temp2=zeros(m1,m2);
m_temp3=zeros(m1,m2);
m_temp4=zeros(m1,m2);
m_temp5=zeros(m1,m2);

index=[];

cur=length(index)+1;
while length(index) <= nrep
   
    if cases(c,2)==0
        w=1.0*randn(n,1);
    else
        w=1.0*rndlaplace(n,1);
    end
       
    if cases(c,1)==0
        x_e= 0.5*w + sqrt(sdx)*randn(n,1);  
    else
        x_e= 0.5*w + sqrt(sdx)*rndlaplace(n,1);
    end
    
    
y_e=true_g(x_e,cases(c,4)).*true_g(w,cases(c,5));

if cases(c,3)==0
    x=x_e+sdx*randn(n,1);
    z=x_e+sdx*randn(n,1);
else
    x=x_e+sdx*rndlaplace(n,1);
    z=x_e+sdx*rndlaplace(n,1);
end


u=true_g(w,cases(c,5)) + 0.3*randn(n,1); 

y = true_g(x_e,cases(c,4)).*u;
E_y = mean(y);
SD_y = std(y);
y=(true_g(x_e,cases(c,4)).*u -E_y)./SD_y; % standardized to have mean zero and s.d. one

if cases(c,4)==0
    true_beta = ( 2/sqrt(pi)*exp(-thex^2).*Ey_x(ys2,lz,dz,thew))./SD_y;
else
    true_beta = ( (thex >= -1)*(thex <=1).*Ey_x(ys2,lz,dz,thew))./SD_y;
end


for i=1:m1
    h1=h1_vec(i);
    h3=h1-6.5;
    h5=h1-6.5;
    h7=h1;
    h9=h1;
    
    for ii=1:m2
        h2=h2_vec(ii);
        h4=h2-2;
        h6=h2-2;
        h8=h2;
        h10=h2;
    
    [f_x1, muhat1, betahat1]=nlme(y,x,z,w,h1,h2,nb,lz,dz,thew);
    yh1=Ey_x(betahat1,lz,dz,thex); %Fourier 1
    
    [f_x4, muhat4, betahat4]=nlme(y,x_e,z,w,h7,h8,nb,lz,dz,thew);
    yh4=Ey_x(betahat4,lz,dz,thex); %Fourier 2
    
    [f_x5, muhat5, betahat5]=nlme(y,z,x_e,w,h9,h10,nb,lz,dz,thew);
    yh5=Ey_x(betahat5,lz,dz,thex); %Fourier 3

    kx_e =  ftkernel2(x_e-thex,h3);
    kw_e =  ftkernel2(w-thew,h4);
    
    kx =  ftkernel2(x-thex,h5);
    kw =  ftkernel2(w-thew,h6);
    
    k1 = calc_diag(n,h3,h4,kx_e,kw_e);
    k2 = calc_diag(n,h5,h6,kx,kw);
    
    
    chi1 = [ones(n,1) (x_e-thex) (w-thew)];
    chi2 = [ones(n,1) (x-thex) (w-thew)];
    yh2=inv(chi1'*k1*chi1)*chi1'*k1*y; %local linear estimator without measurement error
    yh3=inv(chi2'*k2*chi2)*chi2'*k2*y; %local linear estimator with measurement error
    
    if sum(isnan(yh1))> 0 | max(abs(yh1)') > 50 | sum(isnan(yh4))> 0 | max(abs(yh4)') > 50 ...
        | sum(isnan(yh5))> 0 | max(abs(yh5)') > 50 ...
        | sum(isnan(yh2(2)))> 0 | max(abs(yh2(2))') > 50 | sum(isnan(yh3(2)))> 0 | max(abs(yh3(2))') > 50
    cur=cur+1;
  
    continue
    end

    b_temp1(i,ii)= b_temp1(i,ii)+yh1/nrep;
    v_temp1(i,ii)= v_temp1(i,ii)+yh1^2/nrep;
    m_temp1(i,ii)= m_temp1(i,ii)+(yh1 - true_beta)^2/nrep;
    
    b_temp4(i,ii)= b_temp4(i,ii)+yh4/nrep;
    v_temp4(i,ii)= v_temp4(i,ii)+yh4^2/nrep;
    m_temp4(i,ii)= m_temp4(i,ii)+(yh4 - true_beta)^2/nrep;
    
    b_temp5(i,ii)= b_temp5(i,ii)+yh5/nrep;
    v_temp5(i,ii)= v_temp5(i,ii)+yh5^2/nrep;
    m_temp5(i,ii)= m_temp5(i,ii)+(yh5 - true_beta)^2/nrep;
    
    
    b_temp2(i,ii)= b_temp2(i,ii)+yh2(2)/nrep;
    v_temp2(i,ii)= v_temp2(i,ii)+yh2(2)^2/nrep;
    m_temp2(i,ii)= m_temp2(i,ii)+(yh2(2) - true_beta)^2/nrep;
    b_temp3(i,ii)= b_temp3(i,ii)+yh3(2)/nrep;
    v_temp3(i,ii)= v_temp3(i,ii)+yh3(2)^2/nrep;
    m_temp3(i,ii)= m_temp3(i,ii)+(yh3(2) - true_beta)^2/nrep;
    
    end
end

index=[index;cur];

cur=cur+1;

end

bias_f1 = (b_temp1 - true_beta).^2;
var_f1 = v_temp1 - b_temp1.^2;
rmse_f1 = sqrt(bias_f1+var_f1);

bias_f2 = (b_temp2 - true_beta).^2;
var_f2 = v_temp2 - b_temp2.^2;
rmse_f2 = sqrt(bias_f2+var_f2);

bias_f3 = (b_temp3 - true_beta).^2;
var_f3 = v_temp3 - b_temp3.^2;
rmse_f3 = sqrt(bias_f3+var_f3);

bias_f4 = (b_temp4 - true_beta).^2;
var_f4 = v_temp4 - b_temp4.^2;
rmse_f4 = sqrt(bias_f4+var_f4);

bias_f5 = (b_temp5 - true_beta).^2;
var_f5 = v_temp5 - b_temp5.^2;
rmse_f5 = sqrt(bias_f5+var_f5);


toc
